import re
from mext.uri.schemes import *

__all__ = ['urlsplit']




def urlsplit(url):
    """
        Parse an URL as <scheme>://<netloc></path>?<query>#<fragment>
        Return (scheme, netloc, path, query, fragment).
    """
    #url = unicode(url)
    key = type(url), url
    if key in _parse_cache:
        try:
            return _parse_cache[key]
        except:
            # catch cache cleanup from a different thread
            pass

    if len(_parse_cache) >= MAX_CACHE_SIZE:
        _clean_cache()

    if ':' in url:
        #if url.startswith('http://'): # optimize the common case
        #    r = ('http',) + _rx_http.match(url[5:]).groups()
        #else:
        scheme, path = url.split(':', 1)
        scheme = scheme.lower()
        #r = _scheme_rx.get(scheme, _rx_default).match(url).groups()
        if scheme in _scheme_rx:
            r = (scheme,) + _scheme_rx[scheme].match(path).groups()
        else:
            r = (scheme, None, path, None, None)
    else:
        r = (None, None) + _rx_relative.match(url).groups()
    _parse_cache[key] = r
    return r





def _gen_stop(sep, terms, qual='*'):
    p = '[^%s]' % ''.join(terms) if terms else '.'
    p += qual
    if sep:
        terms.insert(0, sep)
        #return r'?:\%s(%s)' % (sep, p)
        return r'?:\%s(%s)' % (sep, p)
    else:
        return p

def _generate_pattern(scheme, netloc, query, frag):
    frag_part = query_part = ''
    nl_part = '()'
    terms = []
    if frag:
        frag_part = _gen_stop('#', terms)
    if query:
        query_part = _gen_stop('?', terms)
    path_part = _gen_stop('', terms)
    terms.insert(0, '/')
    if netloc:
        nl_part = _gen_stop('//', terms, '+')[3:]
    #if scheme:
    #    return r'\A%s:%s(%s)(%s)?(%s)?\Z' % (scheme, nl_part, path_part, query_part, frag_part)
    if scheme:
        return r'\A%s(%s)(%s)?(%s)?\Z' % (nl_part, path_part, query_part, frag_part)
    else:
        return r'\A(%s)(%s)?(%s)?\Z' % (path_part, query_part, frag_part)









_scheme_rx = {}

def reg_scheme(scheme):
    if scheme in _scheme_rx:
        return
    pattern = _generate_pattern(scheme,
        scheme in scheme_uses_netloc,
        scheme in scheme_uses_query,
        scheme in scheme_uses_fragment
    )
    _scheme_rx[scheme] = re.compile(pattern)

map(reg_scheme, scheme_uses_netloc)
map(reg_scheme, scheme_uses_query)
map(reg_scheme, scheme_uses_fragment)

_rx_http = _scheme_rx['http']
#_rx_default = re.compile(_generate_pattern('[^:]+', False, False, False))
_rx_relative = re.compile(_generate_pattern('', False, True, True))








##
## urlsplit cache
##

_parse_cache = {}
_clear_half = True
MAX_CACHE_SIZE = 500

def _clean_cache():
    global _clear_half
    keys = list(_parse_cache)
    _clear_half = not _clear_half
    idx_half = len(keys)/2
    if _clear_half:
        keys = keys[:idx_half]
    else:
        keys = keys[idx_half:]
    map(_parse_cache.__delitem__, keys)
    del keys




if __name__ == '__main__':
    print urlsplit('/bar/xxx')
    print urlsplit('/bar/xxx?')
    print urlsplit('/bar/xxx?#')
    print urlsplit('/bar/xxx#')
    print urlsplit('http://foo/bar/xxx')
    print urlsplit('ftp://host/foo/abc')
    print urlsplit('mem:/foo/abc')
    print urlsplit('http://foo:80/bar/xxx')
